# XML-RPC client - Acts as a bridge to windows storage adapters
# Company - Citrix Systems
# Author  - Sankaran Nepolean

import xmlrpclib
import util

class CXSRemoteStorageAdapter:
    def __init__(self):
        self.proxy = xmlrpclib.ServerProxy("http://10.95.0.51:10041/")
        return

    def enumerateStorageSystems(self, cred, filters, list):
        try:
            list = self.proxy.enumerageStorageSystems(cred)
            util.SMlog("DEBUG:Storage System List" %list)  
        except xmlrpclib.ProtocolError, err:
            return -1

    def getStorageSystemInfo(self, cred, ssid, ssInfo):
        try:
            resp = self.proxy.getStorageSystemInfo(cred, ssid)
            util.SMlog("DEBUG: SS info "%ssInfo)
            prepareStorageSystemInfo(resp, ssInfo)
        except xmlrpclib.ProtocolError, err:
            return -1

    def enumerateStoragePools(self, cred, ssid, filters, list):
        try:
            resp = self.proxy.enumerateStoragePools(cred, ssid)
        except xmlrpclib.ProtocolError, err:
            return -1

    def getStoragePoolInfo(self, cred, ssid, pool, spInfo):
        try:
            resp = self.proxy.getStoragePoolInfo(cred, ssid, pool)
            util.SMlog("DEBUG: SS info "%spInfo)
            prepareStoragePoolInfo(resp, spInfo)
        except xmlrpclib.ProtocolError, err:
            return -1

    def enumerateStorageNodes(self, cred, ssid, spid, filters, list):
        try:
            resp = self.proxy.enumerateStorageNodes(cred, ssid, spid)
        except xmlrpclib.ProtocolError, err:
            return -1

    def getStorageVolumeInfo(self, cred, ssid, svid, svInfo):
        try:
            resp = self.proxy.getStorageNodeInfo(cred, ssid, svid)
            prepareStorageNodeInfo(resp, svInfo)
        except xmlrpclib.ProtocolError, err:
            return -1

    def createStorageNode(self, cred, name, ssid, spid, size, raid_type, prov_type, prov_opt, group_name, svInfo):
        try:
            resp = self.proxy.enumerateStoragePools(cred, ssid)
            resp = self.proxy.createStorageNode(cred, name, ssid, spid, size, group_name)
            prepareStorageNodeInfo(resp, svInfo)
        except xmlrpclib.ProtocolError, err:
            return -1

    def resizeStorageNode(self, cred, ssid, svid, size, svInfo):
        try:
            resp = self.proxy.enumerateStorageNodes(cred, ssid, "")
            resp = self.proxy.resizeStorageNode(cred, ssid, svid, size)
        except xmlrpclib.ProtocolError, err:
            return -1

    def deleteStorageNode(self, cred, svid):
        try:
            resp = self.proxy.deleteStorageNode(cred, svid)
        except xmlrpclib.ProtocolError, err:
            return -1

    def assignStorageNode(self, cred, ssid, svid, hostname, initiatorList, listLen, viewId, chapUser, chapPass, hostType, protType, SAList):
        try:
            initList = [] 
            for count in range(initiatorList.size()):
                initList.append(initiatorList[count])
            resp = self.proxy.assignStorageNode(cred, ssid, svid, hostname, initList)
        except xmlrpclib.ProtocolError, err:
            return -1

    def unassignStorageNode(self, cred, ssid, svid, hostname, initiatorList, viewId, protType):
        try:
            resp = self.proxy.enumerateStorageNodes(cred, ssid, "")
            resp = self.proxy.unassignStorageNode(cred, ssid, svid, hostname)
        except xmlrpclib.ProtocolError, err:
            return -1

    def enumerateTargetPorts(self, cred, ssid, flag, filters, TargetInfoList):
       try:
            resp = self.proxy.enumerateTargetPorts(cred, ssid)
            prepareTargetInfo(resp, TargetInfoList)
       except xmlrpclib.ProtocolError, err:
            return -1

    def setupSnapshot(self, cred, cred1, sessionInfo, ssaInfoList):
        try:
            util.SMlog("DEBUG: setup before")
            sessInfo = prepareSnapSessionInfo(sessionInfo)
            resp = self.proxy.enumerateStorageNodes(cred, sessInfo['sourceStorageSystemId'], "")
            resp = self.proxy.setupSnapshot(cred, sessInfo)
            prepareSnapAssocInfo(resp[0], ssaInfoList)
        except xmlrpclib.ProtocolError, err:
            return -1

    def createSnapshot(self, cred, cred1, sessionInfo, ssaInfoList, len):
        try:
            sessInfo = prepareSnapSessionInfo(sessionInfo)
            resp = self.proxy.enumerateStorageNodes(cred, sessInfo['sourceStorageSystemId'], "")
            ssInfo = {}
            ssInfo['sourceStorageVolumeId'] = ssaInfoList[0].sourceStorageVolumeId;
            resp = self.proxy.createSnapshot(cred, sessInfo, ssInfo)
            prepareSnapAssocInfo(resp, ssaInfoList)
        except xmlrpclib.ProtocolError, err:
            return -1

def xstr(str):
    if str is None:
        return ''
    else:
        return str

def prepareSnapSessionInfo(sessionInfo):
    sessInfo = {}
    sessInfo['sessionId'] = xstr(sessionInfo.sessionId)
    sessInfo['sourceStorageSystemId'] = xstr(sessionInfo.sourceStorageSystemId)
    #sessInfo['sourceStorageVolumeIdList'] = []
    #sessInfo['sourceStorageVolumeIdList'].append(xstr(sessionInfo.sourceStorageVolumeIdList.pop_front()))
    sessInfo['sourceStorageVolumeId'] = xstr(sessionInfo.sourceStorageVolumeIdList[0])
    sessInfo['targetStorageSystemId'] = xstr(sessionInfo.targetStorageSystemId)
    #sessInfo['targetStorageVolumeIdList'] = []
    #sessInfo['targetStorageVolumeIdList'].append(xstr(sessionInfo.targetStorageVolumeIdList.pop_front()))
    sessInfo['targetStorageVolumeId'] = xstr(sessionInfo.targetStorageVolumeIdList[0])
    #sessInfo['targetStorageVolumeBackingStoreList'] = []
    #sessInfo['targetStorageVolumeBackingStoreList'].append(xstr(sessionInfo.targetStorageVolumeBackingStoreList.pop_front()))
    sessInfo['targetStorageVolumeBackingStore'] = xstr(sessionInfo.targetStorageVolumeBackingStoreList[0])
    sessInfo['snapshotOperation'] = xstr(sessionInfo.snapshotOperation)
    sessInfo['snapshotType'] = xstr(sessionInfo.snapshotType)
    sessInfo['provisioningType'] = xstr(sessionInfo.provisioningType)
    sessInfo['snapshotState'] = xstr(sessionInfo.snapshotState)
    sessInfo['targetRaidLevel'] = xstr(sessionInfo.targetRaidLevel)
    sessInfo['targetStoragePoolId'] = xstr(sessionInfo.targetStoragePoolId)
    #sessInfo['targetFriendlyNameList'] = []
    #sessInfo['targetFriendlyNameList'].append(xstr(sessionInfo.targetFriendlyNameList.pop_front()))
    sessInfo['targetFriendlyName'] = xstr(sessionInfo.targetFriendlyNameList[0])
    util.SMlog("DEBUG: inside preparesession sessionInfo: %s" %sessInfo)

    return sessInfo


def prepareTargetInfo(resp, TargetInfoList):
    util.SMlog("DEBUG: Count of Target Ports %d" %len(TargetInfoList))
    util.SMlog("DEBUG: Count of Target Ports %s" % len(resp))
    util.SMlog("DEBUG: Target Ports %s" %resp)
    TargetInfoList.resize(len(resp))
    for count in range(len(resp)):
        TargetInfoList[count].storageSystemId = resp[count]['ssid']
        TargetInfoList[count].portId = resp[count]['tportid']
        TargetInfoList[count].portIqn = resp[count]['tportiqn']
        TargetInfoList[count].friendlyName = resp[count]['frname']
        TargetInfoList[count].protocolType = int(resp[count]['prottype'], 16)
        TargetInfoList[count].connectionType = resp[count]['conntype']
    util.SMlog("DEBUG: Target List ssid %s" %TargetInfoList[0].portIqn)
    util.SMlog("DEBUG: Target List ssid %s" %TargetInfoList[0].portId)
    util.SMlog("DEBUG: Target List ssid %s" %TargetInfoList[0].protocolType)
        
def prepareStorageSystemInfo(resp, ssInfo):
    ssInfo.storageSystemId = resp['storage_system_id']
    ssInfo.friendlyName = resp['friendly_name']
    ssInfo.serialNum = resp['serial_num']
    ssInfo.vendor = resp['vendor']
    ssInfo.model = resp['model']
    ssInfo.capabilities = int(resp['capabilities'])
    for count in range(int(resp['Snapshot'])):
       ssInfo.snapshotMethods[count].name = resp['SnapshotInfo'][count]['Name']
       ssInfo.snapshotMethods[count].snapshotType = resp['SnapshotInfo'][count]['Type']
       ssInfo.snapshotMethods[count].snapshotCapabilities = resp['SnapshotInfo'][count]['Cap']
       ssInfo.snapshotMethods[count].nodeTypes = resp['SnapshotInfo'][count]['Targ']
       ssInfo.snapshotMethods[count].maxSnapshots = resp['SnapshotInfo'][count]['Max']

def prepareStoragePoolInfo(resp, spInfo):
    spInfo.friendlyName = resp['friendly_name']
    spInfo.storagePoolId = resp['storage_pool_id']
    spInfo.storageSystemId = resp['storage_system_id']
    spInfo.parentPool = resp['parent_pool']
    spInfo.sizeInMB = long(resp['sizeMB'])
    spInfo.freeSpaceInMB = long(resp['freeMB'])
    spInfo.availableFreeSpaceInMB = long(resp['freeMB'])
    spInfo.supportedRaidTypes = long(resp['supported_raid_types'], 16)
    spInfo.supportedNodeTypes = long(resp['supported_node_types'], 16)
    spInfo.supportedProvisioningTypes = long(resp['supported_prov_types'], 16)
    spInfo.isDefault = (0, 1) [resp['is_default'] == "TRUE"]

def prepareStorageNodeInfo(resp, svInfo):
    if type(resp) is dict:
        svInfo.storageSystemId = resp['storage_system_id']
        svInfo.storageVolumeId = resp['storage_node_id']
        svInfo.storagePoolId = resp['storage_pool_id']
        svInfo.friendlyName = resp['friendly_name']
        svInfo.serialNum = resp['serial_num']
        svInfo.usedSpaceInMB = long(resp['usedMB'])
        svInfo.sizeInMB = long(resp['sizeMB'])
        svInfo.status = int(resp['status'])
        svInfo.snapshotFlags = int(resp['is_snap'])
        if resp.has_key('LunIqn'):
            svInfo.lunIqn = resp['LunIqn']
        svInfo.nodeType = int(resp['node_type'], 16)
        svInfo.accessPermissions = int(resp['access_perm'], 16)
        svInfo.raidLevel = resp['raid_level']
        svInfo.groupId = resp['group_id']

def prepareSnapAssocInfo(resp, snapInfo):
    snapInfo.resize(1)
    if type(resp) is dict:
        util.SMlog("DEBUG: setup after")
        #To-Do - InComplete
        snapInfo[0].targetStorageVolumeId = resp['tgt_id']
    util.SMlog("DEBUG: setup after")

#Utility Functions
def ListOfStrings():
    return [] 

def ListOfStorageSysInfo():
    return []
